-- https://doi.org/10.1002/sia.740130202 p.8

CREATE SEQUENCE seq_experiments NOCACHE;

CREATE TABLE experiments
(
    id                                                         bigint                                   NOT NULL default nextval(seq_experiments),
    format_identifier                                          VARCHAR(255)                             NOT NULL,
    institution_identifier                                     VARCHAR(255)                             NOT NULL,
    instrument_model_identifier                                VARCHAR(255)                             NOT NULL,
    operator_identifier                                        VARCHAR(255)                             NOT NULL,
    experiment_identifier                                      text                                     NOT NULL,
    number_of_lines_in_comment                                 INT                                      NOT NULL,
    comment                                                    text                                     NOT NULL COMMENT 'The number of occurrences of comment line is specified by the value of number of lines in comment above. The comment may include details of the last calibration of the instrument',
    experiment_mode                                            ENUM ('MAP', 'SDP', 'MAPDP', 'NORM')     NOT NULL,
    scan_mode                                                  ENUM ('REGULAR', 'IRREGULAR', 'MAPPING') NOT NULL,
    number_of_spectral_regions                                 INT                                      NOT NULL COMMENT 'Normally only one technique is used in an experiment but there may be more. The value of number of spectral regions is the sum for all techniques of the numbers of spectral regions in each technique. number of spectral regions is inserted if and only if the value of experiment mode is MAP, MAPDP, NORM or SDP',
    number_of_analysis_positions                               INT                                      NOT NULL,
    number_of_discrete_x_coordinates_in_full_map               INT                                      NOT NULL,
    number_of_discrete_y_coordinates_in_full_map               INT                                      NOT NULL,
    number_of_experimental_variables                           INT                                      NOT NULL,
    number_of_entries_in_parameter_inclusion_or_exclusion_list INT                                      NOT NULL COMMENT 'If the values of any of the block parameters are the same in all of the blocks their values may be sent in the first block and then omitted from all subsequent blocks. If number of entries in parameter inclusion or exclusion list is positive then the parameters listed are to be included; if it is negative then the parameters listed are to be excluded; if it is zero then all parameters are to be given in all blocks.',
    number_of_manually_entered_items_in_block                  INT                                      NOT NULL,
    number_of_future_upgrade_experiment_entries                INT                                      NOT NULL,
    number_of_future_upgrade_block_entries                     INT                                      NOT NULL,
    number_of_blocks                                           INT                                      NOT NULL,
    experiment_terminator                                      CHAR(17)                                 NOT NULL DEFAULT 'end of experiment',
    PRIMARY KEY (id)
) WITH SYSTEM VERSIONING COMMENT 'VAMAS Experiment Data';

CREATE SEQUENCE seq_experiments_variables NOCACHE;

CREATE TABLE experiments_variables
(
    id            bigint       NOT NULL DEFAULT NEXTVAL(seq_experiments_variables),
    experiment_id BIGINT       NOT NULL,
    label         VARCHAR(255) NOT NULL,
    unit          TEXT         NOT NULL,
    FOREIGN KEY (experiment_id) REFERENCES experiments (id)
)
    WITH SYSTEM VERSIONING COMMENT 'An experimental variable is a parameter which may be varied from block to block through the experiment but which remains constant within each block';

CREATE SEQUENCE seq_experiments_parameter_inclusion_or_exclusion_prefix_numbers NOCACHE;

CREATE TABLE experiments_parameter_inclusion_or_exclusion_prefix_numbers
(
    id            bigint NOT NULL DEFAULT NEXTVAL(seq_experiments_parameter_inclusion_or_exclusion_prefix_numbers),
    experiment_id BIGINT NOT NULL,
    number        BIGINT NOT NULL,
    FOREIGN KEY exp_par_inc_or_exc_pre_num_1 (experiment_id) REFERENCES experiments (id)
)
    WITH SYSTEM VERSIONING COMMENT 'The number of occurrences of parameter inclusion or exclusion prefix number is given by the absolute value of number of entries in parameter inclusion or exclusion list above. If this is greater than zero then the values of successive occurrences of parameter inclusion or exclusion prefix number should be in ascending order. The values of parameter inclusion or exclusion prefix number refer to the numbers in comment brackets given below to the left of the parameters in the syntax-rule defining block and must be within the range of these numbers';

CREATE SEQUENCE seq_experiments_prefix_numbers_of_manually_entered_items NOCACHE;

CREATE TABLE experiments_prefix_numbers_of_manually_entered_items
(
    id            bigint NOT NULL DEFAULT NEXTVAL(seq_experiments_prefix_numbers_of_manually_entered_items),
    experiment_id BIGINT NOT NULL,
    prefix_number TEXT   NOT NULL,
    FOREIGN KEY exp_pre_num_of_man_ent_ite_1 (experiment_id) REFERENCES experiments (id)
)
    WITH SYSTEM VERSIONING COMMENT 'The number of occurrences of prefix number of manually entered item is specified by the value of number of manually entered items in block above. If this is greater than zero then the values of successive occurrences of prefix number of manually entered item should be in ascending order. Any of the items preceded by prefix numbers in comment brackets in the syntax-rule defining block which need to be evaluated by the operator and manually entered from the keyboard should be included in this list. If an item is to be expressed as a real number and the operator is unable tosupply a value then the computer should enter the value 1E37';

CREATE SEQUENCE seq_experiments_future_upgrade_experiment_entries NOCACHE;

CREATE TABLE experiments_future_upgrade_experiment_entries
(
    id            bigint NOT NULL DEFAULT NEXTVAL(seq_experiments_future_upgrade_experiment_entries),
    experiment_id BIGINT NOT NULL,
    entry         TEXT   NOT NULL,
    FOREIGN KEY exp_fut_upg_exp_ent_1 (experiment_id) REFERENCES experiments (id)
)
    WITH SYSTEM VERSIONING COMMENT 'The number of occurrences of future upgrade experiment entry is given by the value of number of future upgrade experiment entries above. It is defined as a text line so that any integer, real number or text line inserted here by a future upgrade of the Format can be read as a text line then discarded';

CREATE SEQUENCE seq_block NOCACHE;

CREATE TABLE blocks
(
    experiment_id                                              BIGINT                                                                                                                                                      NOT NULL,
    block_identifier                                           VARCHAR(255)                                                                                                                                                NOT NULL,
    sample_identifier                                          TEXT                                                                                                                                                        NOT NULL,
    date                                                       DATE                                                                                                                                                        NOT NULL,
    number_of_hours_in_advance_of_greenwich_mean_time          INT                                                                                                                                                         NOT NULL,
    number_of_lines_in_block_comment                           INT                                                                                                                                                         NOT NULL,
    comment                                                    TEXT                                                                                                                                                        NOT NULL COMMENT 'The number of occurrences of comment line is specified by the value of number of lines in block comment above.',
    technique                                                  ENUM ('AES diff','AES dir', 'EDX', 'ELS', 'FABMS', 'FABMS energy spec', 'ISS', 'SIMS', 'SIMS energy spec', 'SNMS', 'SNMS energy spec', 'UPS', 'XPS', 'XRF') NOT NULL,
    x_coordinate                                               DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'The ordinal number, starting with unity, of the point in the array along the analysis source deflection system x-axis',
    y_coordinate                                               DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'The ordinal number, starting with unity, of the point in the array along the analysis source deflection system y-axis',
    analysis_source_label                                      VARCHAR(255)                                                                                                                                                NOT NULL,
    sputtering_ion_or_atomic_number                            INT                                                                                                                                                         NOT NULL,
    number_of_atoms_in_sputtering_ion_or_atom_particle         INT                                                                                                                                                         NOT NULL,
    sputtering_ion_of_atom_charge_sign_and_number              INT                                                                                                                                                         NOT NULL,
    analysis_source_characteristic_energy                      DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Energy in electron volts',
    analysis_source_strength                                   DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Power in watts for XPS and XRF, beam current in nanoamps for AES, EDX, ISS, SIMS and SNMS; beam equivalent for FABMS',
    analysis_source_beam_width_x                               DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Width in micrometres at the sample in the plane perpendicular to the source beam',
    analysis_source_beam_width_y                               DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Width in micrometres at the sample in the plane perpendicular to the source beam',
    field_of_view_x                                            DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Micrometres',
    field_of_view_y                                            DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Micrometres',
    first_linescan_start_x_coordinate                          INT                                                                                                                                                         NOT NULL,
    first_linescan_start_y_coordinate                          INT                                                                                                                                                         NOT NULL,
    first_linescan_finish_x_coordinate                         INT                                                                                                                                                         NOT NULL,
    first_linescan_finish_y_coordinate                         INT                                                                                                                                                         NOT NULL,
    last_linescan_finish_x_coordinate                          INT                                                                                                                                                         NOT NULL,
    last_linescan_finish_y_coordinate                          INT                                                                                                                                                         NOT NULL,
    analysis_source_polar_angle_of_incidence                   VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees from upward z-direction, defined by the sample stage',
    analysis_source_azimuth                                    VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees clockwise from the y-direction towards the operator, defined by the sample stage',
    analyser_mode                                              ENUM ('FAT', 'FRR', 'constant delta m', 'constant m/delta m')                                                                                               NOT NULL COMMENT 'Degrees clockwise from the y-direction towards the operator, defined by the sample stage',
    analyser_pass_energy_or_retard_ratio_or_mass_resolution    DECIMAL(20, 8) COMMENT 'Energy in electron volts, mass in amu',
    differential_width                                         INT                                                                                                                                                         NOT NULL COMMENT 'Electron volts peak-to-peak for sinusoidal modulation or computer differentiation. Differential width is inserted if and only if the value of technique is AES diff.',
    magnification_of_analyser_transfer_lens                    VARCHAR(255)                                                                                                                                                NOT NULL,
    analyser_work_function_or_acceptance_energy_of_atom_or_ion DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Positive value for work function in electron volts for AES, XPS, UPS, ELS and ISS. The acceptance energy of an ion is the energy filter pass energy of the mass spectrometer for SIMS, SNMS and FABMS',
    target_bias                                                VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Target bias is in volts, including the sign',
    analysis_width_x                                           VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'The analysis width x is the gated signal width of the source in the x-direction in the plane perpendicular to the beam for FABMS, FABMS energy spec, ISS, SIMS, SIMS energy spec, SNMS and SNMS energy spec, the analyser slit length divided by the magnification of the analyser transfer lens to that slit for AES diff, AES dir, ELS, UPS and XPS, and is the source width in the x-direction for both EDX and XRF analysis width x is in micrometres',
    analysis_width_y                                           VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'As analysis width x but for y',
    analyser_axis_take_off_polar_angle                         VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees from upward z-direction, defined by the sample stage',
    analyser_axis_take_off_azimuth                             VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees clockwise from the y-direction towards the operator, defined by the sample stage',
    species_label                                              VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Elemental symbol or molecular formula',
    transition_or_charge_state_label                           VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'For example, KLL for AES, 1s for XPS, -1 for SIMS',
    charge_of_detected_particle                                INT                                                                                                                                                         NOT NULL COMMENT 'For example, -1 for AES and XPS, +1 for positive SIMS',
    abscissa_label                                             VARCHAR(255)                                                                                                                                                NOT NULL,
    abscissa_units                                             VARCHAR(255)                                                                                                                                                NOT NULL,
    abscissa_start                                             DECIMAL(20, 8)                                                                                                                                              NOT NULL,
    abscissa_increment                                         DECIMAL(20, 8)                                                                                                                                              NOT NULL,
    number_of_corresponding_variables                          INT                                                                                                                                                         NOT NULL,
    # 32, blocks_corresponding_variable
    signal_mode                                                ENUM ('analogue', 'pulse counting')                                                                                                                         NOT NULL COMMENT 'Analogue signals, while recorded digitally, may be of either sign and have a gain which may be noted in the block comment. Pulse counting signals are integers with values equal to or greater than zero',
    signal_collection_time                                     DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Time in seconds per scan for each channel or may-point, except for both EDX and XRF where it is the total spectrum collection time',
    number_of_scans_to_compile_this_block                      INT                                                                                                                                                         NOT NULL,
    signal_time_correction                                     VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Signal time correction is in seconds',
    sputtering_source_energy                                   DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Energy in electron volts',
    sputtering_source_beam_current                             DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Current in nanoamps or equivalent for neutrals',
    sputtering_source_width_x                                  DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Width in micrometres at the sample in the plane perpendicular to the sputtering source beam',
    sputtering_source_width_y                                  DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Width in micrometres at the sample in the plane perpendicular to the sputtering source beam',
    sputtering_source_polar_angle_of_incidence                 DECIMAL(20, 8)                                                                                                                                              NOT NULL COMMENT 'Degrees from upward z-direction, defined by the sample stage',
    sputtering_source_azimuth                                  DECIMAL(20, 8)                                                                                                                                              NULL COMMENT 'Degrees clockwise from the y-direction towards the operator, defined by the sample stage',
    sputtering_mode                                            ENUM ('continuous', 'cyclic')                                                                                                                               NULL,
    sample_normal_polar_angle_of_tilt                          VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees from upward z-direction, defined by the sample stage',
    sample_normal_tilt_azimuth                                 VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees clockwise from the y-direction towards the operator, defined by the sample stage',
    sample_rotation_angle                                      VARCHAR(255)                                                                                                                                                NOT NULL COMMENT 'Degrees clockwise rotation about the sample normal. If this is referenced to a particular direction on the sample this direction would be specified in a block comment line.',
    number_of_additional_numerical_parameters                  INT                                                                                                                                                         NOT NULL,
    # 40, blocks_additional_numerical_parameter
    number_of_ordinate_values                                  INT                                                                                                                                                         NOT NULL COMMENT 'The value of number of ordinate values is equal to product of the value of number of corresponding variables and the number of sets of corresponding variables to be transferred.',
    # blocks_blocks_min_max_ordinate_values
    # blocks_ordinate_values
    # blocks_additional_numerical_parameters
    analyser_pass_energy_of_retard_ratio_or_mass_resolution    DECIMAL(20, 8)                                                                                                                                              NOT NULL,
    carriage_return                                            BIT(7)                                                                                                                                                      NULL,
    PRIMARY KEY (block_identifier),
    FOREIGN KEY (experiment_id) REFERENCES experiments (id)
) WITH SYSTEM VERSIONING;

CREATE SEQUENCE seq_blocks_values_of_experimental_variables NOCACHE;

CREATE TABLE blocks_values_of_experimental_variables
(
    id       BIGINT       NOT NULL DEFAULT NEXTVAL(seq_blocks_values_of_experimental_variables),
    block_id VARCHAR(255) NOT NULL,
    variable VARCHAR(255) NOT NULL,
    PRIMARY KEY (id),
    FOREIGN KEY fk_blo_val_of_exp_var_1 (block_id) REFERENCES blocks (block_identifier)
)
    WITH SYSTEM VERSIONING COMMENT 'Value of experimental variable may be, for example, total time in seconds, total etch time in seconds, temperature in Kelvin, energy in electron volts or mass in unified atomic mass units';

CREATE SEQUENCE seq_blocks_corresponding_variables NOCACHE;

CREATE TABLE blocks_corresponding_variables
(
    id       BIGINT       NOT NULL DEFAULT NEXTVAL(seq_blocks_corresponding_variables),
    block_id VARCHAR(255) NOT NULL,
    label    VARCHAR(255) NOT NULL,
    unit     TEXT         NOT NULL,
    PRIMARY KEY (id),
    FOREIGN KEY fk_blo_cor_var_lab_1 (block_id) REFERENCES blocks (block_identifier)
)
    WITH SYSTEM VERSIONING;

CREATE SEQUENCE seq_blocks_additional_numerical_parameters NOCACHE;

CREATE TABLE blocks_additional_numerical_parameters
(
    id       BIGINT         NOT NULL DEFAULT NEXTVAL(seq_blocks_additional_numerical_parameters),
    block_id VARCHAR(255)   NOT NULL,
    label    VARCHAR(255)   NOT NULL,
    unit     TEXT           NOT NULL,
    value    DECIMAL(20, 8) NOT NULL,
    PRIMARY KEY (id),
    FOREIGN KEY fk_blo_add_num_par_lab_1 (block_id) REFERENCES blocks (block_identifier)
)
    WITH SYSTEM VERSIONING;

CREATE SEQUENCE seq_blocks_future_update_bock_entries NOCACHE;

CREATE TABLE blocks_future_update_block_entries
(
    id       BIGINT       NOT NULL DEFAULT NEXTVAL(seq_blocks_future_update_bock_entries),
    block_id VARCHAR(255) NOT NULL,
    entry    TEXT         NOT NULL COMMENT 'Any integer, real number or text line inserted at this point by a future upgrade of the Format can be read as a text line then discarded',
    PRIMARY KEY (id),
    FOREIGN KEY fk_blo_fut_upd_blo_ent_1 (block_id) REFERENCES blocks (block_identifier)
)
    WITH SYSTEM VERSIONING;

CREATE SEQUENCE seq_blocks_min_max_ordinate_values NOCACHE;

CREATE TABLE blocks_min_max_ordinate_values
(
    id       BIGINT         NOT NULL DEFAULT NEXTVAL(seq_blocks_min_max_ordinate_values),
    block_id VARCHAR(255)   NOT NULL,
    minimum  DECIMAL(20, 8) NOT NULL,
    maximum  DECIMAL(20, 8) NOT NULL,
    PRIMARY KEY (id),
    FOREIGN KEY fk_blo_min_max_ord_val_1 (block_id) REFERENCES blocks (block_identifier)
)
    WITH SYSTEM VERSIONING;

CREATE SEQUENCE seq_blocks_ordinate_values NOCACHE;

CREATE TABLE blocks_ordinate_values
(
    id       BIGINT         NOT NULL DEFAULT NEXTVAL(seq_blocks_ordinate_values),
    position INT            NOT NULL,
    block_id VARCHAR(255)   NOT NULL,
    value    DECIMAL(20, 8) NOT NULL,
    PRIMARY KEY (id),
    FOREIGN KEY fk_blo_min_ord_val_1 (block_id) REFERENCES blocks (block_identifier)
)
    WITH SYSTEM VERSIONING;
